@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2008 the original author or authors.
+ * Copyright 2002-2009 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,10 +16,13 @@
 
 package net.sf.json;
 
+import java.lang.ref.SoftReference;
 import java.util.HashSet;
 import java.util.Iterator;
 import java.util.Set;
 
+import net.sf.json.util.JSONTokener;
+import net.sf.json.util.JSONUtils;
 import net.sf.json.util.JsonEventListener;
 
 import org.apache.commons.logging.Log;
@@ -31,13 +34,22 @@ import org.apache.commons.logging.LogFactory;
  * @author Andres Almiray <aalmiray@users.sourceforge.net>
  */
 abstract class AbstractJSON {
-   // private static Set cycleSet = new HashSet();
-
-   private static ThreadLocal cycleSet = new ThreadLocal(){
-      protected synchronized Object initialValue() {
-         return new HashSet();
+   private static class CycleSet extends ThreadLocal {
+      protected Object initialValue() {
+         return new SoftReference(new HashSet());
       }
-   };
+
+      public Set getSet() {
+         Set set = (Set) ((SoftReference)get()).get();
+         if( set == null ) {
+             set = new HashSet();
+             set(new SoftReference(set));
+         }
+         return set;
+      }
+   }
+   
+   private static CycleSet cycleSet = new CycleSet();
 
    private static final Log log = LogFactory.getLog( AbstractJSON.class );
 
@@ -207,7 +219,59 @@ abstract class AbstractJSON {
       getCycleSet().remove( instance );
    }
 
+   protected Object _processValue( Object value, JsonConfig jsonConfig ) {
+      if( JSONNull.getInstance().equals( value ) ) {
+         return JSONNull.getInstance();
+      } else if( Class.class.isAssignableFrom( value.getClass() ) || value instanceof Class ) {
+         return ((Class) value).getName();
+      } else if( JSONUtils.isFunction( value ) ) {
+         if( value instanceof String ) {
+            value = JSONFunction.parse( (String) value );
+         }
+         return value;
+      } else if( value instanceof JSONString ) {
+         return JSONSerializer.toJSON( (JSONString) value, jsonConfig );
+      } else if( value instanceof JSON ) {
+         return JSONSerializer.toJSON( value, jsonConfig );
+      } else if( JSONUtils.isArray( value ) ) {
+         return JSONArray.fromObject( value, jsonConfig );
+      } else if( JSONUtils.isString( value ) ) {
+         String str = String.valueOf( value );
+         if( JSONUtils.hasQuotes( str ) ){
+            str = JSONUtils.stripQuotes( str );
+            if( JSONUtils.isFunction( str )){
+               return JSONUtils.DOUBLE_QUOTE + str + JSONUtils.DOUBLE_QUOTE;
+            }
+            return str;
+         } else if( JSONUtils.isJsonKeyword( str, jsonConfig ) ) {
+            if( jsonConfig.isJavascriptCompliant() && "undefined".equals( str )){
+               return JSONNull.getInstance();
+            }
+            return str;
+         } else if( JSONUtils.mayBeJSON( str ) ) {
+            try {
+               return JSONSerializer.toJSON( str, jsonConfig );
+            } catch( JSONException jsone ) {
+               return str;
+            }
+         }
+         return str;
+      } else if( JSONUtils.isNumber( value ) ) {
+         JSONUtils.testValidity( value );
+         return JSONUtils.transformNumber( (Number) value );
+      } else if( JSONUtils.isBoolean( value ) ) {
+         return value;
+      } else {
+         JSONObject jsonObject = JSONObject.fromObject( value, jsonConfig );
+         if( jsonObject.isNullObject() ) {
+            return JSONNull.getInstance();
+         } else {
+            return jsonObject;
+         }
+      }
+   }
+   
    private static Set getCycleSet() {
-      return (Set) cycleSet.get();
+      return cycleSet.getSet();
    }
 }
\ No newline at end of file
