@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2008 the original author or authors.
+ * Copyright 2002-2009 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,6 +16,7 @@
 package net.sf.json;
 
 import java.io.Writer;
+import java.io.Serializable;
 
 /**
  * Marker interface, identifies a valid JSON value.<br>
@@ -24,7 +25,7 @@ import java.io.Writer;
  *
  * @author Andres Almiray <aalmiray@users.sourceforge.net>
  */
-public interface JSON {
+public interface JSON extends Serializable {
    /**
     * Returns true if this object is a JSONArray, false otherwise.
     */
