@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2007 the original author or authors.
+ * Copyright 2002-2009 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,30 +15,6 @@
  */
 package net.sf.json;
 
-/*
-Copyright (c) 2002 JSON.org
-
-Permission is hereby granted, free of charge, to any person obtaining a copy
-of this software and associated documentation files (the "Software"), to deal
-in the Software without restriction, including without limitation the rights
-to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-copies of the Software, and to permit persons to whom the Software is
-furnished to do so, subject to the following conditions:
-
-The above copyright notice and this permission notice shall be included in all
-copies or substantial portions of the Software.
-
-The Software shall be used for Good, not Evil.
-
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
-SOFTWARE.
-*/
-
 import java.beans.PropertyDescriptor;
 import java.io.IOException;
 import java.io.Writer;
@@ -49,11 +25,13 @@ import java.lang.reflect.ParameterizedType;
 import java.lang.reflect.Type;
 import java.util.ArrayList;
 import java.util.Collection;
+import java.util.ConcurrentModificationException;
 import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
 import java.util.ListIterator;
 import java.util.Map;
+import java.util.NoSuchElementException;
 import java.util.Set;
 
 import net.sf.ezmorph.Morpher;
@@ -693,7 +671,6 @@ public final class JSONArray extends AbstractJSON implements JSON, List, Compara
     * @param array An boolean[] array.
     */
    private static JSONArray _fromArray( boolean[] array, JsonConfig jsonConfig ) {
-      fireArrayStartEvent( jsonConfig );
       if( !addInstance( array ) ){
          try{
             return jsonConfig.getCycleDetectionStrategy()
@@ -709,10 +686,11 @@ public final class JSONArray extends AbstractJSON implements JSON, List, Compara
             throw jsone;
          }
       }
+      fireArrayStartEvent( jsonConfig );
       JSONArray jsonArray = new JSONArray();
       for( int i = 0; i < array.length; i++ ){
          Boolean b = array[i] ? Boolean.TRUE : Boolean.FALSE;
-         jsonArray.elements.add( b );
+         jsonArray.addValue( b, jsonConfig );
          fireElementAddedEvent( i, b, jsonConfig );
       }
 
@@ -727,7 +705,6 @@ public final class JSONArray extends AbstractJSON implements JSON, List, Compara
     * @param array An byte[] array.
     */
    private static JSONArray _fromArray( byte[] array, JsonConfig jsonConfig ) {
-      fireArrayStartEvent( jsonConfig );
       if( !addInstance( array ) ){
          try{
             return jsonConfig.getCycleDetectionStrategy()
@@ -743,10 +720,11 @@ public final class JSONArray extends AbstractJSON implements JSON, List, Compara
             throw jsone;
          }
       }
+      fireArrayStartEvent( jsonConfig );
       JSONArray jsonArray = new JSONArray();
       for( int i = 0; i < array.length; i++ ){
          Number n = JSONUtils.transformNumber( new Byte( array[i] ) );
-         jsonArray.elements.add( n );
+         jsonArray.addValue( n, jsonConfig );
          fireElementAddedEvent( i, n, jsonConfig );
       }
 
@@ -761,7 +739,6 @@ public final class JSONArray extends AbstractJSON implements JSON, List, Compara
     * @param array An char[] array.
     */
    private static JSONArray _fromArray( char[] array, JsonConfig jsonConfig ) {
-      fireArrayStartEvent( jsonConfig );
       if( !addInstance( array ) ){
          try{
             return jsonConfig.getCycleDetectionStrategy()
@@ -777,10 +754,11 @@ public final class JSONArray extends AbstractJSON implements JSON, List, Compara
             throw jsone;
          }
       }
+      fireArrayStartEvent( jsonConfig );
       JSONArray jsonArray = new JSONArray();
       for( int i = 0; i < array.length; i++ ){
          Character c = new Character( array[i] );
-         jsonArray.elements.add( c );
+         jsonArray.addValue( c, jsonConfig );
          fireElementAddedEvent( i, c, jsonConfig );
       }
 
@@ -795,7 +773,6 @@ public final class JSONArray extends AbstractJSON implements JSON, List, Compara
     * @param array An double[] array.
     */
    private static JSONArray _fromArray( double[] array, JsonConfig jsonConfig ) {
-      fireArrayStartEvent( jsonConfig );
       if( !addInstance( array ) ){
          try{
             return jsonConfig.getCycleDetectionStrategy()
@@ -811,12 +788,13 @@ public final class JSONArray extends AbstractJSON implements JSON, List, Compara
             throw jsone;
          }
       }
+      fireArrayStartEvent( jsonConfig );
       JSONArray jsonArray = new JSONArray();
       try{
          for( int i = 0; i < array.length; i++ ){
             Double d = new Double( array[i] );
             JSONUtils.testValidity( d );
-            jsonArray.elements.add( d );
+            jsonArray.addValue( d, jsonConfig );
             fireElementAddedEvent( i, d, jsonConfig );
          }
       }catch( JSONException jsone ){
@@ -837,8 +815,6 @@ public final class JSONArray extends AbstractJSON implements JSON, List, Compara
     * @throws JSONException If there is a syntax error.
     */
    private static JSONArray _fromArray( Enum e, JsonConfig jsonConfig ) {
-
-      fireArrayStartEvent( jsonConfig );
       if( !addInstance( e ) ){
          try{
             return jsonConfig.getCycleDetectionStrategy()
@@ -854,9 +830,10 @@ public final class JSONArray extends AbstractJSON implements JSON, List, Compara
             throw jsone;
          }
       }
+      fireArrayStartEvent( jsonConfig );
       JSONArray jsonArray = new JSONArray();
       if( e != null ){
-         jsonArray.elements.add( e.toString() );
+         jsonArray.addValue( e, jsonConfig );
          fireElementAddedEvent( 0, jsonArray.get( 0 ), jsonConfig );
       }else{
          JSONException jsone = new JSONException( "enum value is null" );
@@ -876,7 +853,6 @@ public final class JSONArray extends AbstractJSON implements JSON, List, Compara
     * @param array An float[] array.
     */
    private static JSONArray _fromArray( float[] array, JsonConfig jsonConfig ) {
-      fireArrayStartEvent( jsonConfig );
       if( !addInstance( array ) ){
          try{
             return jsonConfig.getCycleDetectionStrategy()
@@ -892,12 +868,13 @@ public final class JSONArray extends AbstractJSON implements JSON, List, Compara
             throw jsone;
          }
       }
+      fireArrayStartEvent( jsonConfig );
       JSONArray jsonArray = new JSONArray();
       try{
          for( int i = 0; i < array.length; i++ ){
             Float f = new Float( array[i] );
             JSONUtils.testValidity( f );
-            jsonArray.elements.add( f );
+            jsonArray.addValue( f, jsonConfig );
             fireElementAddedEvent( i, f, jsonConfig );
          }
       }catch( JSONException jsone ){
@@ -917,7 +894,6 @@ public final class JSONArray extends AbstractJSON implements JSON, List, Compara
     * @param array An int[] array.
     */
    private static JSONArray _fromArray( int[] array, JsonConfig jsonConfig ) {
-      fireArrayStartEvent( jsonConfig );
       if( !addInstance( array ) ){
          try{
             return jsonConfig.getCycleDetectionStrategy()
@@ -933,10 +909,11 @@ public final class JSONArray extends AbstractJSON implements JSON, List, Compara
             throw jsone;
          }
       }
+      fireArrayStartEvent( jsonConfig );
       JSONArray jsonArray = new JSONArray();
       for( int i = 0; i < array.length; i++ ){
          Number n = new Integer( array[i] );
-         jsonArray.elements.add( n );
+         jsonArray.addValue( n, jsonConfig );
          fireElementAddedEvent( i, n, jsonConfig );
       }
 
@@ -951,7 +928,6 @@ public final class JSONArray extends AbstractJSON implements JSON, List, Compara
     * @param array An long[] array.
     */
    private static JSONArray _fromArray( long[] array, JsonConfig jsonConfig ) {
-      fireArrayStartEvent( jsonConfig );
       if( !addInstance( array ) ){
          try{
             return jsonConfig.getCycleDetectionStrategy()
@@ -967,10 +943,11 @@ public final class JSONArray extends AbstractJSON implements JSON, List, Compara
             throw jsone;
          }
       }
+      fireArrayStartEvent( jsonConfig );
       JSONArray jsonArray = new JSONArray();
       for( int i = 0; i < array.length; i++ ){
          Number n = JSONUtils.transformNumber( new Long( array[i] ) );
-         jsonArray.elements.add( n );
+         jsonArray.addValue( n, jsonConfig );
          fireElementAddedEvent( i, n, jsonConfig );
       }
 
@@ -982,8 +959,6 @@ public final class JSONArray extends AbstractJSON implements JSON, List, Compara
    // ------------------------------------------------------
 
    private static JSONArray _fromArray( Object[] array, JsonConfig jsonConfig ) {
-      fireArrayStartEvent( jsonConfig );
-
       if( !addInstance( array ) ){
          try{
             return jsonConfig.getCycleDetectionStrategy()
@@ -999,6 +974,7 @@ public final class JSONArray extends AbstractJSON implements JSON, List, Compara
             throw jsone;
          }
       }
+      fireArrayStartEvent( jsonConfig );
       JSONArray jsonArray = new JSONArray();
       try{
          for( int i = 0; i < array.length; i++ ){
@@ -1028,7 +1004,6 @@ public final class JSONArray extends AbstractJSON implements JSON, List, Compara
     * @param array An short[] array.
     */
    private static JSONArray _fromArray( short[] array, JsonConfig jsonConfig ) {
-      fireArrayStartEvent( jsonConfig );
       if( !addInstance( array ) ){
          try{
             return jsonConfig.getCycleDetectionStrategy()
@@ -1044,10 +1019,11 @@ public final class JSONArray extends AbstractJSON implements JSON, List, Compara
             throw jsone;
          }
       }
+      fireArrayStartEvent( jsonConfig );
       JSONArray jsonArray = new JSONArray();
       for( int i = 0; i < array.length; i++ ){
          Number n = JSONUtils.transformNumber( new Short( array[i] ) );
-         jsonArray.elements.add( n );
+         jsonArray.addValue( n, jsonConfig );
          fireElementAddedEvent( i, n, jsonConfig );
       }
 
@@ -1057,8 +1033,6 @@ public final class JSONArray extends AbstractJSON implements JSON, List, Compara
    }
 
    private static JSONArray _fromCollection( Collection collection, JsonConfig jsonConfig ) {
-      fireArrayStartEvent( jsonConfig );
-
       if( !addInstance( collection ) ){
          try{
             return jsonConfig.getCycleDetectionStrategy()
@@ -1074,6 +1048,7 @@ public final class JSONArray extends AbstractJSON implements JSON, List, Compara
             throw jsone;
          }
       }
+      fireArrayStartEvent( jsonConfig );
       JSONArray jsonArray = new JSONArray();
       try{
          int i = 0;
@@ -1099,8 +1074,6 @@ public final class JSONArray extends AbstractJSON implements JSON, List, Compara
    }
 
    private static JSONArray _fromJSONArray( JSONArray array, JsonConfig jsonConfig ) {
-      fireArrayStartEvent( jsonConfig );
-
       if( !addInstance( array ) ){
          try{
             return jsonConfig.getCycleDetectionStrategy()
@@ -1116,11 +1089,12 @@ public final class JSONArray extends AbstractJSON implements JSON, List, Compara
             throw jsone;
          }
       }
+      fireArrayStartEvent( jsonConfig );
       JSONArray jsonArray = new JSONArray();
       int index = 0;
       for( Iterator elements = array.iterator(); elements.hasNext(); ){
          Object element = elements.next();
-         jsonArray.elements.add( element );
+         jsonArray.addValue( element, jsonConfig );
          fireElementAddedEvent( index++, element, jsonConfig );
       }
 
@@ -1134,7 +1108,6 @@ public final class JSONArray extends AbstractJSON implements JSON, List, Compara
    }
 
    private static JSONArray _fromJSONTokener( JSONTokener tokener, JsonConfig jsonConfig ) {
-      fireArrayStartEvent( jsonConfig );
 
       JSONArray jsonArray = new JSONArray();
       int index = 0;
@@ -1143,6 +1116,7 @@ public final class JSONArray extends AbstractJSON implements JSON, List, Compara
          if( tokener.nextClean() != '[' ){
             throw tokener.syntaxError( "A JSONArray text must start with '['" );
          }
+         fireArrayStartEvent( jsonConfig );
          if( tokener.nextClean() == ']' ){
             fireArrayEndEvent( jsonConfig );
             return jsonArray;
@@ -1618,7 +1592,7 @@ public final class JSONArray extends AbstractJSON implements JSON, List, Compara
     * than the length of the JSONArray, then null elements will be added as
     * necessary to pad it out.<br>
     * The string may be a valid JSON formatted string, in tha case, it will be
-    * trabsformed to a JSONArray, JSONObjetc or JSONNull.
+    * transformed to a JSONArray, JSONObject or JSONNull.
     *
     * @param index The subscript.
     * @param value A String value.
@@ -1635,7 +1609,7 @@ public final class JSONArray extends AbstractJSON implements JSON, List, Compara
     * than the length of the JSONArray, then null elements will be added as
     * necessary to pad it out.<br>
     * The string may be a valid JSON formatted string, in tha case, it will be
-    * trabsformed to a JSONArray, JSONObjetc or JSONNull.
+    * transformed to a JSONArray, JSONObject or JSONNull.
     *
     * @param index The subscript.
     * @param value A String value.
@@ -1754,7 +1728,7 @@ public final class JSONArray extends AbstractJSON implements JSON, List, Compara
    /**
     * Append a String value. This increases the array's length by one.<br>
     * The string may be a valid JSON formatted string, in tha case, it will be
-    * trabsformed to a JSONArray, JSONObjetc or JSONNull.
+    * transformed to a JSONArray, JSONObject or JSONNull.
     *
     * @param value A String value.
     * @return this.
@@ -1766,22 +1740,32 @@ public final class JSONArray extends AbstractJSON implements JSON, List, Compara
    /**
     * Append a String value. This increases the array's length by one.<br>
     * The string may be a valid JSON formatted string, in tha case, it will be
-    * trabsformed to a JSONArray, JSONObjetc or JSONNull.
+    * transformed to a JSONArray, JSONObject or JSONNull.
     *
     * @param value A String value.
     * @return this.
     */
    public JSONArray element( String value, JsonConfig jsonConfig ) {
-      if( value == null ){
-         this.elements.add( "" );
-      }else if( JSONUtils.mayBeJSON( value ) ){
+      if( value == null ) {
+         this.elements.add("");
+      } else if( JSONUtils.hasQuotes( value )) {
+         this.elements.add(value);
+      } else if( JSONNull.getInstance().equals( value )) {
+         this.elements.add( JSONNull.getInstance() );
+      } else if( JSONUtils.isJsonKeyword(value,jsonConfig)) {
+         if( jsonConfig.isJavascriptCompliant() && "undefined".equals( value )){
+            this.elements.add( JSONNull.getInstance() );
+         }else{
+            this.elements.add(value);
+         }
+      } else if( JSONUtils.mayBeJSON( value ) ){
          try{
             this.elements.add( JSONSerializer.toJSON( value, jsonConfig ) );
          }catch( JSONException jsone ){
-            this.elements.add( JSONUtils.stripQuotes( value ) );
+            this.elements.add( value );
          }
-      }else{
-         this.elements.add( JSONUtils.stripQuotes( value ) );
+      } else {
+         this.elements.add(value);
       }
       return this;
    }
@@ -2060,7 +2044,7 @@ public final class JSONArray extends AbstractJSON implements JSON, List, Compara
     * Returns an Iterator for this JSONArray
     */
    public Iterator iterator() {
-      return this.elements.iterator();
+      return new JSONArrayListIterator();
    }
 
    /**
@@ -2104,11 +2088,14 @@ public final class JSONArray extends AbstractJSON implements JSON, List, Compara
    }
 
    public ListIterator listIterator() {
-      return elements.listIterator();
+      return listIterator( 0 );
    }
 
    public ListIterator listIterator( int index ) {
-      return elements.listIterator( index );
+      if( index < 0 || index > size() )
+         throw new IndexOutOfBoundsException( "Index: " + index );
+
+      return new JSONArrayListIterator( index );
    }
 
    /**
@@ -2511,56 +2498,20 @@ public final class JSONArray extends AbstractJSON implements JSON, List, Compara
     * @return this.
     */
    private JSONArray _addValue( Object value, JsonConfig jsonConfig ) {
-      this.elements.add( _processValue( value, jsonConfig ) );
+      this.elements.add(value);
       return this;
    }
 
-   private Object _processValue( Object value, JsonConfig jsonConfig ) {
-      if( (value != null && Class.class.isAssignableFrom( value.getClass() ))
-            || value instanceof Class ){
-         return ((Class) value).getName();
-      }else if( JSONUtils.isFunction( value ) ){
-         if( value instanceof String ){
-            value = JSONFunction.parse( (String) value );
-         }
-         return value;
-      }else if( value instanceof JSONString ){
-         return JSONSerializer.toJSON( (JSONString) value, jsonConfig );
-      }else if( value instanceof JSON ){
-         return JSONSerializer.toJSON( value, jsonConfig );
-      }else if( JSONUtils.isArray( value ) ){
-         return JSONArray.fromObject( value, jsonConfig );
-      }else if( value instanceof JSONTokener ){
+   protected Object _processValue( Object value, JsonConfig jsonConfig ) {
+      if( value instanceof JSONTokener ) {
          return _fromJSONTokener( (JSONTokener) value, jsonConfig );
-      }else if( JSONUtils.isString( value ) ){
-         String str = String.valueOf( value );
-         if( JSONUtils.mayBeJSON( str ) ){
-            try{
-               return JSONSerializer.toJSON( str, jsonConfig );
-            }catch( JSONException jsone ){
-               return JSONUtils.stripQuotes( str );
-            }
-         }else{
-            return str;
-         }
-      }else if( JSONUtils.isNumber( value ) ){
-         JSONUtils.testValidity( value );
-         return JSONUtils.transformNumber( (Number) value );
-      }else if( JSONUtils.isBoolean( value ) ){
-         return value;
-      }else if( value instanceof Enum ){
-         return String.valueOf( value );
+      }else if( value != null && Enum.class.isAssignableFrom( value.getClass() ) ){
+         return ((Enum) value).name();
       }else if( value instanceof Annotation || (value != null && value.getClass()
             .isAnnotation()) ){
          throw new JSONException( "Unsupported type" );
-      }else{
-         JSONObject jsonObject = JSONObject.fromObject( value, jsonConfig );
-         if( jsonObject.isNullObject() ){
-            return JSONNull.getInstance();
-         }else{
-            return jsonObject;
-         }
       }
+      return super._processValue( value, jsonConfig );
    }
 
    /**
@@ -2587,4 +2538,89 @@ public final class JSONArray extends AbstractJSON implements JSON, List, Compara
       }
       return _processValue( value, jsonConfig );
    }
+
+   private class JSONArrayListIterator implements ListIterator {
+      int currentIndex = 0;
+      int lastIndex = -1;
+
+      JSONArrayListIterator() {
+
+      }
+
+      JSONArrayListIterator( int index ) {
+         currentIndex = index;
+      }
+
+      public boolean hasNext() {
+         return currentIndex != size();
+      }
+
+      public Object next() {
+         try {
+            Object next = get( currentIndex );
+            lastIndex = currentIndex++;
+            return next;
+         } catch( IndexOutOfBoundsException e ) {
+            throw new NoSuchElementException();
+         }
+      }
+
+      public void remove() {
+         if( lastIndex == -1 )
+            throw new IllegalStateException();
+         try {
+            JSONArray.this.remove( lastIndex );
+            if( lastIndex < currentIndex ){
+               currentIndex--;
+            }
+            lastIndex = -1;
+         } catch( IndexOutOfBoundsException e ) {
+            throw new ConcurrentModificationException();
+         }
+      }
+
+      public boolean hasPrevious() {
+         return currentIndex != 0;
+      }
+
+      public Object previous() {
+         try {
+            int index = currentIndex - 1;
+            Object previous = get( index );
+            lastIndex = currentIndex = index;
+            return previous;
+         } catch( IndexOutOfBoundsException e ) {
+            throw new NoSuchElementException();
+         }
+      }
+
+      public int nextIndex() {
+         return currentIndex;
+      }
+
+      public int previousIndex() {
+         return currentIndex - 1;
+      }
+
+      public void set( Object obj ) {
+         if( lastIndex == -1 ){
+            throw new IllegalStateException();
+         }
+
+         try {
+            JSONArray.this.set( lastIndex, obj );
+         } catch( IndexOutOfBoundsException ex ) {
+            throw new ConcurrentModificationException();
+         }
+      }
+
+      public void add( Object obj ) {
+         try {
+            JSONArray.this.add( currentIndex++, obj );
+            lastIndex = -1;
+         } catch( IndexOutOfBoundsException ex ) {
+            throw new ConcurrentModificationException();
+         }
+      }
+   }
 }
