@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2008 the original author or authors.
+ * Copyright 2002-2009 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -54,7 +54,19 @@ public abstract class NewBeanInstanceStrategy {
          if( target != null ){
             Constructor c = target.getDeclaredConstructor( EMPTY_PARAM_TYPES );
             c.setAccessible( true );
-            return c.newInstance( EMPTY_ARGS );
+            try {
+               return c.newInstance( EMPTY_ARGS );
+            } catch ( InstantiationException e ) {
+               // getCause() was added on jdk 1.4
+               String cause = "";
+               try { cause = e.getCause() != null ? "\n"+e.getCause().getMessage() : ""; }
+               catch( Throwable t ) { /* ignore */ }
+               throw new InstantiationException(
+                     "Instantiation of \"" +  target + "\" failed. " +
+                     "It's probably because class is an interface, " +
+                     "abstract class, array class, primitive type or void." +
+                     cause );
+            }
          }
          return null;
       }
